@@ -9,6 +9,7 @@ import subprocess
 import sys
 import warnings
 from collections import defaultdict
+from contextlib import suppress
 from dataclasses import replace
 from functools import lru_cache
 from pathlib import Path
@@ -17,6 +18,8 @@ from typing import TYPE_CHECKING, Any, Callable, ClassVar, Literal, TypeVar
 
 from griffe import (
     Alias,
+    AliasResolutionError,
+    CyclicAliasError,
     DocstringAttribute,
     DocstringClass,
     DocstringFunction,
@@ -43,26 +46,35 @@ if TYPE_CHECKING:
 _logger = get_logger(__name__)
 
 
-def _sort_key_alphabetical(item: CollectorItem) -> Any:
-    # chr(sys.maxunicode) is a string that contains the final unicode
-    # character, so if 'name' isn't found on the object, the item will go to
-    # the end of the list.
+def _sort_key_alphabetical(item: CollectorItem) -> str:
+    # `chr(sys.maxunicode)` is a string that contains the final unicode character,
+    # so if `name` isn't found on the object, the item will go to the end of the list.
     return item.name or chr(sys.maxunicode)
 
 
-def _sort_key_source(item: CollectorItem) -> Any:
-    # if 'lineno' is none, the item will go to the start of the list.
+def _sort_key_source(item: CollectorItem) -> float:
+    # If `lineno` is none, the item will go to the end of the list.
     if item.is_alias:
-        return item.alias_lineno if item.alias_lineno is not None else -1
-    return item.lineno if item.lineno is not None else -1
+        return item.alias_lineno if item.alias_lineno is not None else float("inf")
+    return item.lineno if item.lineno is not None else float("inf")
 
 
-Order = Literal["alphabetical", "source"]
-"""Ordering methods."""
+def _sort__all__(item: CollectorItem) -> float:  # noqa: ARG001
+    raise ValueError("Not implemented in public version of mkdocstrings-python")
 
-_order_map = {
+
+Order = Literal["__all__", "alphabetical", "source"]
+"""Ordering methods.
+
+- `__all__`: order members according to `__all__` module attributes, if declared;
+- `alphabetical`: order members alphabetically;
+- `source`: order members as they appear in the source file.
+"""
+
+_order_map: dict[str, Callable[[Object | Alias], str | float]] = {
     "alphabetical": _sort_key_alphabetical,
     "source": _sort_key_source,
+    "__all__": _sort__all__,
 }
 
 
@@ -246,7 +258,7 @@ def do_format_attribute(
 
 def do_order_members(
     members: Sequence[Object | Alias],
-    order: Order,
+    order: Order | list[Order],
     members_list: bool | list[str] | None,
 ) -> Sequence[Object | Alias]:
     """Order members given an ordering method.
@@ -266,7 +278,12 @@ def do_order_members(
             if name in members_dict:
                 sorted_members.append(members_dict[name])
         return sorted_members
-    return sorted(members, key=_order_map[order])
+    if isinstance(order, str):
+        order = [order]
+    for method in order:
+        with suppress(ValueError):
+            return sorted(members, key=_order_map[method])
+    return members
 
 
 # YORE: Bump 2: Remove block.
@@ -396,6 +413,29 @@ def _keep_object(name: str, filters: Sequence[tuple[Pattern, bool]]) -> bool:
     return keep
 
 
+def _parents(obj: Alias) -> set[str]:
+    parent: Object | Alias = obj.parent  # type: ignore[assignment]
+    parents = {obj.path, parent.path}
+    if parent.is_alias:
+        parents.add(parent.final_target.path)  # type: ignore[union-attr]
+    while parent.parent:
+        parent = parent.parent
+        parents.add(parent.path)
+        if parent.is_alias:
+            parents.add(parent.final_target.path)  # type: ignore[union-attr]
+    return parents
+
+
+def _remove_cycles(objects: list[Object | Alias]) -> Iterator[Object | Alias]:
+    suppress_errors = suppress(AliasResolutionError, CyclicAliasError)
+    for obj in objects:
+        if obj.is_alias:
+            with suppress_errors:
+                if obj.final_target.path in _parents(obj):  # type: ignore[arg-type,union-attr]
+                    continue
+        yield obj
+
+
 def do_filter_objects(
     objects_dictionary: dict[str, Object | Alias],
     *,
@@ -455,10 +495,14 @@ def do_filter_objects(
         objects = [
             obj for obj in objects if _keep_object(obj.name, filters) or (inherited_members_specified and obj.inherited)
         ]
-    if keep_no_docstrings:
-        return objects
+    if not keep_no_docstrings:
+        objects = [obj for obj in objects if obj.has_docstrings or (inherited_members_specified and obj.inherited)]
 
-    return [obj for obj in objects if obj.has_docstrings or (inherited_members_specified and obj.inherited)]
+    # Prevent infinite recursion.
+    if objects:
+        objects = list(_remove_cycles(objects))
+
+    return objects
 
 
 @lru_cache(maxsize=1)
