@@ -4,16 +4,26 @@ from __future__ import annotations
 
 import os
 import sys
+from dataclasses import replace
 from glob import glob
+from io import BytesIO
 from pathlib import Path
 from textwrap import dedent
 from typing import TYPE_CHECKING
 
+import mkdocstrings
 import pytest
-from griffe import Docstring, DocstringSectionExamples, DocstringSectionKind, Module, temporary_visited_module
+from griffe import (
+    Docstring,
+    DocstringSectionExamples,
+    DocstringSectionKind,
+    Module,
+    temporary_inspected_module,
+    temporary_visited_module,
+)
 from mkdocstrings import CollectionError
 
-from mkdocstrings_handlers.python import PythonConfig, PythonHandler, PythonOptions
+from mkdocstrings_handlers.python import Inventory, PythonConfig, PythonHandler, PythonOptions
 
 if TYPE_CHECKING:
     from mkdocstrings import MkdocstringsPlugin
@@ -275,3 +285,49 @@ def test_deduplicate_summary_sections(handler: PythonHandler, section: str, code
             ),
         )
         assert html.count(f"{section}:") == 1
+
+
+def test_inheriting_self_from_parent_class(handler: PythonHandler) -> None:
+    """Inspect self only once when inheriting it from parent class."""
+    with temporary_inspected_module(
+        """
+        class A: ...
+        class B(A): ...
+        A.B = B
+        """,
+    ) as module:
+        # Assert no recusrion error.
+        handler.render(
+            module,
+            handler.get_options({"inherited_members": True}),
+        )
+
+
+def test_specifying_inventory_base_url(handler: PythonHandler) -> None:
+    """Assert that the handler renders inventory URLs using the specified base_url."""
+    # Update handler config to include an inventory with a base URL
+    base_url = "https://docs.com/my_library"
+    inventory = Inventory(url="https://example.com/objects.inv", base_url=base_url)
+    handler.config = replace(handler.config, inventories=[inventory])
+
+    # Mock inventory bytes
+    item_name = "my_library.my_module.MyClass"
+    mocked_inventory = mkdocstrings.Inventory()
+    mocked_inventory.register(
+        name=item_name,
+        domain="py",
+        role="class",
+        uri=f"api-reference/#{item_name}",
+        dispname=item_name,
+    )
+    mocked_bytes = BytesIO(mocked_inventory.format_sphinx())
+
+    # Get inventory URL and config
+    url, config = handler.get_inventory_urls()[0]
+
+    # Load the mocked inventory
+    _, item_url = next(handler.load_inventory(mocked_bytes, url, **config))
+
+    # Assert the URL is based on the provided base URL
+    msg = "Expected inventory URL to start with base_url"
+    assert item_url.startswith(base_url), msg
